import java.io.*;
import java.lang.*;

public class Map {
	public static final int width = 100;
	public static final int height = 100;
	public static final int maxSpawn = 16;
	
	public static final int spawnP1 = 1;
	public static final int spawnP2 = 2;
	public static final int spawnDM = 3;
	
	/* Тайлы карты [z][x][y]
	 * z = 0 -- Фоновая текстура
	 * z = 1 -- Переднеплановая текстура
	 * z = 2 -- Тип тайла
	 */
	public static byte tile[][][];
	public static int spawnx, spawny;
	public static Texture sky;
	
	private static void setFlatIndex(int t, int i, int z) {
		int y = i / width;
		int x = i - y * width;
		tile[z][x][y] = (byte) t;
	}
	
	private static void loadTextures(File fp, int size) throws Exception {
		int num = size / 9; // 9 байт на структуру
		Texman.resetWalls(num);
		for(int i = 0; i < num; i++) {
			String name = fp.readFixedString(8);
			byte type = fp.readByte();
			Texman.loadWall(name, type, i);
		}
	}
	
	private static void loadTiles(File fp, int size, int z) throws IOException {
		int j = 0;
		for(int i = 0; i < size; i++) {
			int id = fp.read();
			if(id == 0xff) {
				int len = fp.readShort();
				id = fp.read();
				for(int k = j; k < j + len; k++)
					setFlatIndex(id, k, z);
				i += 3;
				j += len;
			} else {
				setFlatIndex(id, j, z);
				++j;
			}
		}
	}
	
	private static void loadThings(File fp, int size) throws IOException {
		int len = size / 8; // 8 байт на структуру
		for(int i = 0; i < len; i++) {
			short x = fp.readShort();
			short y = fp.readShort();
			short type = fp.readShort();
			short flag = fp.readShort();
			
			if(type == spawnP1) {
				spawnx = x;
				spawny = y;
			}
		}
	}
	
	private static void loadMusic(File fp, int size) throws IOException {
		String name = fp.readFixedString(8);
		System.out.println("Load music " + name);
		// TODO : здесь должна быть загрузка музыки
	}
	
	private static void loadSky(File fp, int size) throws Exception {
		short id = fp.readShort();
		String name = "RSKY" + id;
		sky = Texman.loadTexture(name, false);
	}
	
	private static void loadSwitch(File fp, int size) throws IOException {
		int num = size / 9; // 9 байт на структуру
		for(int i = 0; i < num; i++) {
			int x = fp.read();
			int y = fp.read();
			int type = fp.read();
			int tm = fp.read(); // наверно бесполезно
			int a = fp.read();
			int b = fp.read();
			int c = fp.readShort(); // не используется
			int flag = fp.read();
			// TODO : здесь должна быть загрузка триггера
		}
	}

	public static void loadMap(int num) throws Exception {
		String name = "MAP" + ((num <= 9) ? ('0' + Integer.toString(num)) : Integer.toString(num));
		File fp = Resource.openLump(name);
		
		/* Читаем заголовок */
		byte[] sig = { 'D', 'o', 'o', 'm', '2', 'D', 0x1a, 0x00 };
		byte[] head = new byte[sig.length];
		fp.read(head);
		for(int i = 0; i < sig.length; i++)
			if(head[i] != sig[i])
				throw new Exception(name + " is not an Doom2D map");
		
		short version = fp.readShort();
		if(version != 2)
			throw new Exception(name + ": unsupported version " + version);
		
		/* Чистим память */
		tile = new byte[3][width][height];
		
		/* Читаем кишки */
		boolean end = false;
		while(!end) {
			int type = fp.readShort();
			int sub  = fp.readShort();
			int size = fp.readInt();
			
			switch(type) {
			case 0: end = true;             break;
			case 1: loadTextures(fp, size); break;
			case 2: loadTiles(fp, size, 0); break;
			case 3: loadTiles(fp, size, 2); break;
			case 4: loadTiles(fp, size, 1); break;
			case 5: loadThings(fp, size);   break;
			case 7: loadMusic(fp, size);    break;
			case 8: loadSky(fp, size);      break;
			case 9: loadSwitch(fp, size);   break;
			default:
				System.out.println(name + ": waring: unknown block type " + type);
				fp.skip(size);
			}
		}
		
		fp.close();
	}
}